
"""Plugin interface."""


from zope.interface import implements

from twisted.python import reflect
from twisted.plugin import IPlugin, getPlugins

from mzbot import ibot, plugins


def listConfigPlugins():
    """Return an iterable of strings describing available plugins."""
    for plugin in getPlugins(ibot.IBotConfigPlugin, package=plugins):
        reply = plugin.doc
        if not reply:
            reply = u'undocumented'
        if not (reply.startswith(plugin.name) and ':' in reply):
            reply = '%s: %s' % (plugin.name, reply)
        yield reply


def getConfigPluginByName(name):
    """Return a plugin object by name or None if it does not exist."""
    handlerPlugins = [
        plugin for plugin in getPlugins(ibot.IBotConfigPlugin,
                                        package=plugins)
        if plugin.name == name]
    if not handlerPlugins:
        return None
    if len(handlerPlugins) > 1:
        raise ValueError('multiple plugins with that name')
    return handlerPlugins[0]


class ConfigPlugin(object):

    """Build a single plugin from a python object."""

    implements(IPlugin, ibot.IBotConfigPlugin)

    def __init__(self, name, importpath, doc):
        self.name = name
        self.importpath = importpath
        self.doc = doc

    def buildHandler(self, *args):
        return reflect.namedAny(self.importpath)(*args)
