
"""Spell check handler."""


from zope.interface import implements

import enchant

from mzbot import commands, ebot, ibot


class SpellHandler(commands.CommandHandler, commands.ConfigurableMixin):

    """Spell check words."""

    implements(ibot.IMessageHandler)

    configurables = {
        u'maxsuggestions': (
            'The maximum number of suggestions per word.',
            'maxSuggestions', u'%d', commands.integer),
        }

    def __init__(self, language='en', maxSuggestions=5):
        commands.CommandHandler.__init__(self)
        self.language = language
        self.maxSuggestions = maxSuggestions
        self.dict = enchant.Dict(self.language)

    @commands.args(commands.stringlist)
    def handle_spell(self, user, source, words):
        """spell word1 [word2 ...]: spellcheck words."""
        if len(words) == 1:
            word = words[0]
            if self.dict.check(word):
                source.msgTo(user, u'"%s" is spelled correctly.' % (word,))
            else:
                suggest = self.dict.suggest(word)[:self.maxSuggestions]
                if suggest:
                    source.msgTo(user, u'%s: %s' % (word, u', '.join(suggest)))
                else:
                    source.msgTo(user,
                                 u'You just made "%s" up, right?' % (word,))
            return
        misspelled = list(word for word in words if not self.dict.check(word))
        if not misspelled:
            source.msgTo(user, u'All spelled correctly.')
        elif len(misspelled) == 1:
            word = misspelled[0]
            suggest = self.dict.suggest(word)[:self.maxSuggestions]
            if suggest:
                source.msgTo(user, u'%s: %s' % (word, u', '.join(suggest)))
            else:
                source.msgTo(user, u'You just made "%s" up, right?' % (word,))
        else:
            source.msgTo(
                user, u'%s are misspelled, use "spell <word>" for suggestions'
                % (u', '.join(misspelled),))

    @commands.args(commands.string)
    def handle_addword(self, user, source, word):
        """addword <word>: add to the personal wordlist."""
        self.dict.add_to_pwl(word)
        source.msgTo(user, u'added')

    def message(self, user, source, message):
        """Reply to messages with "(sp?)" in them."""
        words = message.split()
        spellwords = []
        for i, word in enumerate(words):
            # Handle both "blah (sp?)" and "blah(sp?)"
            toSpell = None
            if word == '(sp?)' and i:
                toSpell = words[i - 1]
            elif word.endswith('(sp?)'):
                toSpell = word[:-len('(sp?)')]
            # not "is not None", it is '' if message starts with "(sp?)" :)
            if toSpell:
                spellwords.append(toSpell)
        if spellwords:
            try:
                spellwords = [word.decode('utf-8') for word in spellwords]
            except UnicodeError:
                source.msgTo(user, u'I can only spell-check utf-8')
            else:
                self.handle_spell(user, source, u' '.join(spellwords))

    def get_language(self):
        """Get language code."""
        return unicode(self.language.encode('string-escape'), 'ascii')

    def set_language(self, args):
        """Set language code."""
        language, = commands.parseArgs(args, commands.bytestring)
        try:
            self.dict = enchant.Dict(language)
        except enchant.DictNotFoundError:
            raise ebot.InvalidArgument('no dictionary')
        else:
            self.language == language
