

import tempfile

from zope.interface import verify

from twisted.trial import unittest

from mzbot import anagram, ibot, ebot, testutils


class AnagramTest(unittest.TestCase):

    setUp = testutils.setUp

    def _getHandler(self, filename):
        handler = anagram.AnagramHandler()
        handler.handle(self.user, self.user, u'load', filename)
        handler.set(u'addchannel', u'normal')
        return handler

    def testAnagramHandler(self):
        wordFile = tempfile.NamedTemporaryFile()
        filename = unicode(wordFile.name, 'ascii')
        handler = self._getHandler(filename)
        verify.verifyObject(ibot.ICommandHandler, handler)
        for command, args, reply in [
            (u'guess', u'foo', u'start a new anagram game first!'),
            (u'hint', u'', u'start a new anagram game first!'),
            (u'surrender', u'', u'start a new anagram game first!'),
            (u'anagram', u'', u'no words'),
            ]:
            handler.handle(self.user, self.user, command, args)
            self.user.assertEvents(('msgTo', self.user, reply))
        self.assertEqual(u'normal', handler.get(u'channels'))
        self.assertRaises(ebot.InvalidArgument,
                          handler.set, u'removechannel', u'blah')
        handler.set(u'removechannel', u'normal')
        wordFile.write('ab\n')
        wordFile.flush()
        handler = self._getHandler(filename)
        for command, args, reply in [
            (u'anagram', u'', u"Unshuffle this for me please: '\x02ba\x02'"),
            (u'anagram', u'', u"In progress: '\x02..\x02' (remaining letters: "
             u"'\x02ba\x02'). If you're stuck, try 'hint' or 'surrender'."),
            (u'guess', u'ba', u'Wrong, try again!'),
            (u'guess', u'ab', u'Correct! It took you \x022\x02 tries '
             u'and \x020\x02 seconds'),
            (u'anagram', u'', u"Unshuffle this for me please: '\x02ba\x02'"),
            (u'surrender', u'', u"The solution is '\x02ab\x02'"),
            (u'anagram', u'', u"Unshuffle this for me please: '\x02ba\x02'"),
            (u'hint', u'', u"The solution is '\x02ab\x02'"),
            (u'anagram', u'', u"Unshuffle this for me please: '\x02ba\x02'"),
            ]:
            handler.handle(self.user, self.user, command, args)
            self.user.assertEvents(('msgTo', self.user, reply))
        handler.message(self.user, self.user, 'ab')
        handler.message(self.user, self.normal, 'fo')
        handler.message(self.user, self.normal, 'a b')
        self.user.assertEvents()
        self.normal.assertEvents()
        handler.message(self.user, self.normal, 'ab')
        self.normal.assertEvents(
            ('msgTo', self.user,
             u'Correct! It took you \x021\x02 tries and \x020\x02 seconds'))
        wordFile.close()
        wordFile = tempfile.NamedTemporaryFile()
        wordFile.write('aab\naba\n')
        wordFile.flush()
        filename = unicode(wordFile.name, 'ascii')
        handler = self._getHandler(filename)
        handler.handle_anagram(self.user, self.user, u'')
        self.user.assertEvents(
            ('msgTo', self.user,
             u"Unshuffle this for me please: '\x02baa\x02'"))
        handler.handle(self.user, self.user, u'hint', u'')
        self.failUnless(
            self.user.events[0][2] in [
                u"Hint: 'a..' (remaining letters: 'ba')",
                u"Hint: '.a.' (remaining letters: 'ba')",
                u"Hint: '.b.' (remaining letters: 'aa')",
                u"Hint: '..a' (remaining letters: 'ba')",
                u"Hint: '..b' (remaining letters: 'aa')",
                ])
