
"""Tests for corehandlers.py."""


from zope.interface import verify

from twisted.trial import unittest
from twisted.internet import defer

from mzbot import google, ibot, ebot, testutils


data_sporks = '{"responseData": {"results":[{"GsearchResultClass":"GwebSearch","unescapedUrl":"http://en.wikipedia.org/wiki/Spork","url":"http://en.wikipedia.org/wiki/Spork","visibleUrl":"en.wikipedia.org","cacheUrl":"http://www.google.com/search?q\\u003dcache:bNMbCy9HT0oJ:en.wikipedia.org","title":"\\u003cb\\u003eSpork\\u003c/b\\u003e - Wikipedia, the free encyclopedia","titleNoFormatting":"Spork - Wikipedia, the free encyclopedia","content":"A \\u003cb\\u003espork\\u003c/b\\u003e or a foon is a hybrid form of cutlery taking the form of a spoon-like   shallow scoop with three or four fork tines. \\u003cb\\u003eSpork\\u003c/b\\u003e-like utensils, such as the   \\u003cb\\u003e...\\u003c/b\\u003e"},{"GsearchResultClass":"GwebSearch","unescapedUrl":"http://www.sporksf.com/","url":"http://www.sporksf.com/","visibleUrl":"www.sporksf.com","cacheUrl":"http://www.google.com/search?q\\u003dcache:jCny6whG0csJ:www.sporksf.com","title":"\\u003cb\\u003espork\\u003c/b\\u003e restaurant","titleNoFormatting":"spork restaurant","content":"\\u003cb\\u003eSpork\\u003c/b\\u003e Restaurant - 1058 Valencia St., San Francisco CA 94110 - 415.643.5005."},{"GsearchResultClass":"GwebSearch","unescapedUrl":"http://www.spork.org/","url":"http://www.spork.org/","visibleUrl":"www.spork.org","cacheUrl":"http://www.google.com/search?q\\u003dcache:quQCW7u3faUJ:www.spork.org","title":"The Slightly Less Than Official \\u003cb\\u003eSpork\\u003c/b\\u003e Homepage","titleNoFormatting":"The Slightly Less Than Official Spork Homepage","content":"Sep 10, 1996 \\u003cb\\u003e...\\u003c/b\\u003e A \\u003cb\\u003espork\\u003c/b\\u003e is a perfect metaphor for human existance. It tries to function as both   spoon and fork, and because of this dual nature, \\u003cb\\u003e...\\u003c/b\\u003e"},{"GsearchResultClass":"GwebSearch","unescapedUrl":"http://www.thinkgeek.com/interests/giftsforhim/8ace/","url":"http://www.thinkgeek.com/interests/giftsforhim/8ace/","visibleUrl":"www.thinkgeek.com","cacheUrl":"http://www.google.com/search?q\\u003dcache:8YRN84yqs_MJ:www.thinkgeek.com","title":"ThinkGeek :: Titanium \\u003cb\\u003eSpork\\u003c/b\\u003e","titleNoFormatting":"ThinkGeek :: Titanium Spork","content":"Combining the lightweight, strong nature of titanium with the utilitarian,   handiness of the \\u003cb\\u003espork\\u003c/b\\u003e, the perfect eating utensil has been created. \\u003cb\\u003e...\\u003c/b\\u003e"}],"cursor":{"pages":[{"start":"0","label":1},{"start":"4","label":2},{"start":"8","label":3},{"start":"12","label":4},{"start":"16","label":5},{"start":"20","label":6},{"start":"24","label":7},{"start":"28","label":8}],"estimatedResultCount":"128000","currentPageIndex":0,"moreResultsUrl":"http://www.google.com/search?oe\\u003dutf8\\u0026ie\\u003dutf8\\u0026source\\u003duds\\u0026start\\u003d0\\u0026hl\\u003den\\u0026q\\u003dspork"}}, "responseDetails": null, "responseStatus": 200}'


class TestHandler(google.GoogleHandler):

    def __init__(self, page):
        google.GoogleHandler.__init__(self)
        self.page = page

    def _getPage(self, url, headers):
        self.url = url
        self.headers = headers
        return defer.succeed(self.page)


class GoogleHandlerTest(unittest.TestCase):

    setUp = testutils.setUp

    def testInterfaces(self):
        handler = google.GoogleHandler()
        for iface in [ibot.ICommandHandler, ibot.ICommandHelpProvider]:
            verify.verifyObject(iface, handler)

    def test_configurable(self):
        handler = google.GoogleHandler()
        handler.set('add', 'kittens inurl:kittens spork')
        self.assertEqual(u"kittens: 'inurl:kittens spork'",
                         handler.get('additions'))
        self.assertEqual(set(['kittens', 'kittenslist']),
                         handler.commandset)
        self.assertRaises(ebot.InvalidArgument,
                          handler.set, u'add', u'kittens somethingelse')
        handler.set('remove', 'kittens')
        self.assertEqual(u'', handler.get('additions'))
        self.assertRaises(ebot.InvalidArgument,
                          handler.set, u'add', u'kittenslist kittens')
        self.assertRaises(ebot.InvalidArgument,
                          handler.set, u'remove', u'nonexistant')

    def testSearch(self):
        handler = TestHandler(data_sporks)
        handler.set(u'add', u'google')

        handler.handle(self.user, self.user, u'google', u'kittens')
        self.user.assertEvents(
            ('msgTo', self.user, u'http://en.wikipedia.org/wiki/Spork '
             u'(Spork - Wikipedia, the free encyclopedia)'))

        handler.handle(self.user, self.user, u'googlelist', u'kittens')
        self.user.assertEvents(
            ('msgTo', self.user, u'http://en.wikipedia.org/wiki/Spork - '
             u'http://www.sporksf.com/ - http://www.spork.org/ - '
             u'http://www.thinkgeek.com/interests/giftsforhim/8ace/'))

    def testCommandDescription(self):
        handler = google.GoogleHandler()
        handler.set(u'add', u'google')
        self.assertEqual('[target] <query>: do a "" google query',
                         handler.getCommandDescription(u'google'))
        self.assertEqual('[target] <query>: do a "" google query, '
                         'returning the first few results',
                         handler.getCommandDescription(u'googlelist'))
