# do not check docstrings, __init__, missing member
# pylint: disable-msg=C0111, W0232, E1101

"""Tests for quotes.py."""


from zope.interface import verify

from twisted.trial import unittest

from mzbot import quotes, testutils, ibot


class QuotesTest(unittest.TestCase):

    setUp = testutils.setUp

    def testQuoteHandler(self):
        handler = quotes.QuoteHandler()
        verify.verifyObject(ibot.ICommandHandler, handler)
        verify.verifyObject(ibot.IConfigurable, handler)
        self.assertEqual(u'ungrouped', handler.get(u'group'))
        handler.set(u'group', u'\N{SNOWMAN}')
        for command, args, reply in [
            (u'numquotes', u' ', u'0 quotes match'),
            (u'addquote', u'testquote', u'quote 1 added'),
            (u'quote', u'', u'1: testquote'),
            (u'addquote', u'second quote', u'quote 2 added'),
            (u'quote', u'test', u'1: testquote'),
            (u'quote', u'foo',
             u"No matches, if you wanted to add use 'addquote'"),
            (u'numquote', u'2', u'second quote'),
            (u'numquote', u'32', u"I don't have that many quotes"),
            (u'numquote', u'123456789', u"I don't have that many quotes"),
            (u'quotelist', u'q', u"'1: testquote', '2: second quote'"),
            (u'pmquotelist', u'    ', u'need a query'),
            (u'quotelist', u'foosion', u'no matches'),
            (u'pmquotelist', u'foosion', u'no matches'),
            (u'addquote', u'with trailing space  \n', u'quote 3 added'),
            (u'quote', u'trailing', u'3: with trailing space'),
            ]:
            handler.handle(self.user, self.user, command, args)
            self.user.assertEvents(('msgTo', self.user, reply))

        handler.handle_pmquotelist(self.user, self.normal, u'q')
        self.normal.assertEvents()
        self.user.assertEvents(('msg', u'1: testquote'),
                               ('msg', u'2: second quote'))
        for i in range(20):
            handler.handle(self.user, self.user,
                           u'addquote', u'boring %d' % (i,))
        self.user.events = []
        handler.handle_quotelist(self.user, self.user, u'boring')
        self.user.assertEvents(('msgTo', self.user,
                                u'more than 10 matches, refusing to display'))

