# do not check docstrings, __init__, missing member
# pylint: disable-msg=C0111, W0232, E1101

"""Meta-testing."""


from zope.interface import verify

from twisted.trial import unittest
from twisted.internet import defer

from mzbot import testutils, ibot


class TestUtilsTest(unittest.TestCase):

    setUp = testutils.setUp

    def testSetUp(self):
        verify.verifyObject(ibot.IBot, self.bot)
        for channel in [self.empty, self.normal, self.voice, self.oper]:
            verify.verifyObject(ibot.IChannel, channel)
        for channel in [self.normal, self.voice, self.oper]:
            self.failUnless(self.user.nickname in channel.users)
        for user in [self.user, self.other]:
            verify.verifyObject(ibot.IUser, user)

    def testBot(self):
        # caseFolder
        self.assertEquals(self.bot.caseFolder('Foo'), 'foo')
        # getUser
        self.assertIdentical(self.bot.getUser('ME'), self.user)
        newUser = self.bot.getUser('noveluser')
        verify.verifyObject(ibot.IUser, newUser)
        self.assertEquals(newUser.nickname, 'noveluser')
        self.assertIdentical(newUser.bot, self.bot)
        # join
        self.bot.join('foo', 'secret')
        self.bot.join('bar')
        self.bot.assertEvents(('join', 'foo', 'secret'),
                              ('join', 'bar', None))
        # away
        self.bot.away('getting eels')
        self.bot.away()
        self.bot.assertEvents(('away', 'getting eels'),
                              ('away', ''))
        # setNick
        self.bot.setNick('notabot')
        self.bot.assertEvents(('setNick', 'notabot'))
        self.assertEquals(self.bot.user.nickname, 'notabot')
        self.assertEquals(self.bot.user.uname, u'notabot')
        self.assertIdentical(self.bot.users['notabot'], self.bot.user)
        # mode
        self.bot.mode(disabledModes='i')
        self.bot.assertEvents(('mode', '', 'i'))
        # quit
        self.bot.quit('die')
        self.bot.assertEvents(('quit', 'die'))

    def testSource(self):
        for name in ('notice', 'msg', 'me'):
            method = getattr(self.user, name)
            method(u'hi')
            self.user.assertEvents((name, u'hi'))
        self.user.msgTo(self.other, u'hi')
        self.user.assertEvents(('msgTo', self.other, u'hi'))
        self.user.ctcpQuery('PING', 'pong')
        self.user.ctcpReply('PONG', 'ping')
        self.user.assertEvents(('ctcpQuery', 'PING', 'pong'),
                               ('ctcpReply', 'PONG', 'ping'))

    @defer.inlineCallbacks
    def testChannel(self):
        self.failIf(self.empty.modes['b'])
        self.empty.setTopic('hi')
        self.empty.mode(False, 'z')
        self.empty.mode(True, 'b', '*!*@*')
        self.empty.part('too empty')
        self.empty.assertEvents(
            ('topic', 'hi'),
            ('mode', False, 'z', None),
            ('mode', True, 'b', '*!*@*'),
            ('part', 'too empty'))
        self.normal.kick(self.user, 'he sucks')
        self.normal.kick(self.other)
        self.normal.assertEvents(('kick', self.user, 'he sucks'),
                                 ('kick', self.other, None))
        self.normal.remove(self.other, 'he still sucks')
        self.normal.remove(self.other)
        self.normal.assertEvents(('remove', self.other, 'he still sucks'),
                                 ('remove', self.other, None))
        self.assertEqual(u'normal', self.normal.uname)

        # TODO: figure out if this needs to return something well-defined.
        yield self.normal.who()
        self.normal.assertEvents(('who',))

    @defer.inlineCallbacks
    def testUser(self):
        # Make sure attributes are sane.
        complete = testutils.User(
            'complete', realname='Complete',
            username='compl', hostname='complhost')

        self.assertEquals(complete.nickname, 'complete')
        self.assertEquals(complete.realname, 'Complete')
        self.assertEquals(complete.username, 'compl')
        self.assertEquals(complete.hostname, 'complhost')

        # Test who and hostmask a bit.
        who = yield complete.who()
        self.assertIdentical(complete, who)
        hostmask = yield complete.hostmask()
        self.assertEquals(('complete', 'compl', 'complhost'), hostmask)

    def testAssertEvents(self):
        bot = testutils.Bot()
        bot.assertEvents()
        self.assertRaises(unittest.FailTest, bot.assertEvents, 'foo')
        bot.events = [('event', 'random')]
        bot.assertEvents(('event', 'random'))
        bot.events = [('event', 'random')]
        self.assertRaises(unittest.FailTest, bot.assertEvents)
        bot.events = [('event', 'random')]
        self.assertRaises(unittest.FailTest, bot.assertEvents, ('ev', 'a'))
