
"""Test the utility code."""


from twisted.trial import unittest

from mzbot import util


# PreservingFoldingDict and NonPreservingFoldingDict are mostly stolen
# from snakeoil.


class FoldingDictTest(unittest.TestCase):

    def testPreserve(self):
        dct = util.PreservingFoldingDict(
            str.lower, {'Foo':'bar', 'fnz':'donkey'}.iteritems())
        self.assertEqual(dct['fnz'], 'donkey')
        self.assertEqual(dct['foo'], 'bar')
        self.assertEqual(sorted(['bar', 'donkey']), sorted(dct.values()))
        self.assertEqual(dct.copy(), dct)
        self.assertEqual(dct['foo'], dct.get('Foo'))
        self.assert_('foo' in dct)
        keys = ['Foo', 'fnz']
        keysList = list(dct)
        for key in keys:
            self.assert_(key in dct.keys())
            self.assert_(key in keysList)
            self.assert_((key, dct[key]) in dct.items())
        self.assertEqual(len(keys), len(dct))
        self.assert_(dct.pop('foo') == 'bar')
        self.assert_('foo' not in dct)
        del dct['fnz']
        self.assert_('fnz' not in dct)
        dct['Foo'] = 'bar'
        dct.refold(lambda _: _)
        self.assert_('foo' not in dct)
        self.assert_('Foo' in dct)
        self.assertEqual(dct.items(), [('Foo', 'bar')])
        self.assertEqual(list(dct.itervalues()), ['bar'])
        dct.clear()
        self.assertEqual({}, dict(dct))

    def testNoPreserve(self):
        dct = util.NonPreservingFoldingDict(
            str.lower, {'Foo':'bar', 'fnz':'monkey'}.iteritems())
        self.assertEqual(sorted(['bar', 'monkey']), sorted(dct.values()))
        self.assertEqual(dct.copy(), dct)
        keys = ['foo', 'fnz']
        keysList = [key for key in dct]
        for key in keys:
            self.assert_(key in dct.keys())
            self.assert_(key in dct)
            self.assert_(key in keysList)
            self.assert_((key, dct[key]) in dct.items())
        self.assertEqual(len(keys), len(dct))
        self.assertEqual(dct.pop('foo'), 'bar')
        self.assertEqual(list(dct.itervalues()), ['monkey'])
        del dct['fnz']
        self.assertEqual(dct.keys(), [])
        dct.clear()
        self.assertEqual({}, dict(dct))
