

from __future__ import absolute_import, with_statement, division

from twisted.trial import unittest

# Make sure we fail if yaml is importable but mzbot.yaml is not.
try:
    import yaml
    yaml # shut up pyflakes
except ImportError: # pragma: no cover
    yaml = None
else:
    from mzbot import yaml


class YAMLTest(unittest.TestCase):

    if yaml is None: # pragma: no cover
        skip = 'pyyaml not available'

    def _dump(self, string, data):
        """Check data serializes to string and deserializes back to data."""
        dumped = yaml.dump(data)
        self.assertEqual(string, dumped)
        self.assertEqual(data, yaml.load(string))

    def test_dump_str_ascii(self):
        self._dump('!!binary |\n  YQ==\n', 'a')

    def test_dump_unicode_ascii(self):
        self._dump('a\n', u'a')

    test_dump_unicode_ascii.todo = 'is this terminator actually necessary?'

    def test_dump_list_unicode_ascii(self):
        self._dump('[a]\n', [u'a'])

    def test_dump_str_utf8(self):
        self._dump('!!binary |\n  4piD\n', u'\N{SNOWMAN}'.encode('utf-8'))

    def test_dump_str_other(self):
        self._dump('!!binary |\n  /w==\n', '\xff')

    def test_dump_unicode_nonascii(self):
        self._dump('"\\u2603"\n', u'\N{SNOWMAN}')

    def _load(self, data, string):
        """Check that loading string produces data.

        It explicitly compares the types too.
        """
        result = yaml.load(string)
        self.assertEqual(data, result)
        self.assertEqual(type(data), type(result))

    def test_load_str_ascii(self):
        self._load(u'a', 'a\n')

    def test_load_str_utf8(self):
        self._load(u'\N{SNOWMAN}', u'\N{SNOWMAN}\n'.encode('utf-8'))

    def test_load_binary_ascii(self):
        self._load('a', '!!binary YQ==\n')

    def test_load_binary_utf8(self):
        self._load(u'\N{SNOWMAN}'.encode('utf-8'), '!!binary 4piD\n')

    def test_load_binary_other(self):
        self._load('\xff', '!!binary /w==\n')
