
"""Play a game. It has wolves."""


from __future__ import division, absolute_import

import random
import operator
from collections import defaultdict

from zope.interface import implements

from twisted.internet import defer

from mzbot import commands, ibot


PHASE_NONE, PHASE_STARTING, PHASE_NIGHT, PHASE_LYNCH = xrange(4)


# TODO: relay messages between wolves.
# TODO: handle timeouts expiring while off-channel
# (possibly by never having them while off-channel)


GOAT = u'{0} has been kicked by a goat. (by {1})'

PING_TOO_FAST = u"Don't be a spammer :("

PING_ALIVE = u'Please say something to prove you are alive, {0}'

KILL_IDLE = u'{0} died from not getting out of bed!'

STATS = u'''\
There are now {wolves:d} wolves, {seers:d} seers and {harlots:d} harlots'''

VOTES = u'''\
There are now {0:d} players, {1:d} votes are needed to lynch and \
{2:d} players are available to vote.'''

BAD_TARGET = u'Try {0} someone who is actually alive and playing.'

STARTED = u'''\
{0} has started a game. Type !join to join now. \
{0}, when you are ready type !start to start the game.'''

WAIT_FOR_PLAYERS = u'''\
Please wait at least {0:d} more seconds for other players to join.'''

NOW_NIGHT = u'''\
It is now nighttime. Everyone check for PMs from me \
with instructions, if you do not have a PM from me, \
just sit back and relax until morning.'''

NOW_LYNCH = u'''\
You must now vote for who to lynch. \
Type '!lynch <nickname>' to cast your vote.'''

YOU_ARE_SEER = u'''\
You are a seer. \
It is your job to use your power to see who is a wolf \
and guide the villagers to lynch them. \
Further instructions will be provided, \
but please keep your identity a secret.'''

SEER_SEE = u'''\
You are a seer. Now, you must choose someone to see. \
Type '!see nickname' here, where nickname is one of the other players. \
Remember, it is your job to find the wolves!'''

CANNOT_MULTI_SEE = u'''\
You can't see again tonight. Wait til tomorrow.'''

YOU_ARE_WOLF = u'''\
You are a wolf. \
It is your job to kill all the villagers. \
Further instructions will be provided, \
but please keep your identity a secret.'''

WOLF_KILL = u'''\
You are a wolf. HOOOOOOWL. \
Now, you must choose your victim. \
Type '!kill nickname' here, where nickname is one of the other players.'''

YOU_ARE_HARLOT = u'''\
You are the village harlot. \
Each night, you may choose one other player to visit. \
You may type '!visit <nickname>' at any time during the night \
to choose a friend to visit. \
If you choose the wolf, or the wolf visits your friend, you will be killed. \
'''

HARLOT_VISIT = u'''\
You are a harlot. \
Now, you must choose someone to visit. \
Type '!visit nickname' here, where nickname is one of \
the other players.'''

HARLOT_VISIT_SELF = u"Don't be ridiculous."

HARLOT_VISITS_ALREADY = u'You already chose a friend for tonight.'

HARLOT_VISITS_TARGET = u'''\
You are spending the night with {0}. Have a lovely evening!'''

HARLOT_VISITS_YOU = u'''\
You are spending the night with {0}, the village harlot. \
Have a lovely evening!'''

HARLOT_VISITS_WOLF = u'''\
The harlot, {0}, made the unfortunate choice of visiting a wolf last night \
and is now dead.'''

HARLOT_VISITS_VICTIM = u'''\
The village harlot, {0}, made the unfortunate choice of visiting the victim \
last night and is now dead.'''

YOU_ARE_SHOOTER = u'''\
You have {0:d} silver bullet(s). \
This bullet has special properties... \
it can kill a wolf instantly, but will only wound a non-wolf. \
You may type '!shoot <nickname>' at any time during the day \
to use your power.'''

SHOOTER_SHOT = u'{0} shot {1} with a silver bullet!'
SHOOTER_SHOT_VILLAGER = u'''\
{0} is not a wolf, and is injured but will likely have a full recovery. \
They will be resting and unable to vote for one day.'''
SHOOTER_KILLED_VILLAGER = u'''\
The good news is {0} is not a wolf. \
The bad news is {1} shot them in the head. Aim lower next time.'''
SHOOTER_SHOT_WOLF = u'{0} is a wolf, and is dying from the silver bullet.'
SHOOTER_MISSED = u'{0} is a lousy shot. They missed!'
SHOOTER_KILLED_SELF = u'''\
{0} should clean their weapon more often. It has exploded and killed them!'''
SHOOTER_SHOT_SELF = u'{0} is a lousy shot. They hit their own foot!'

WOLVES_MISSED_HARLOT = u'''\
The wolves tried to kill the harlot... who wasn't home'''

WOLVES_KILLED = u'''\
The villagers awake, thankful to have survived the night. \
But one did not! Upon searching the village, the body of {0} {1} \
is found. Those remaining mourn {0}'s death.'''

LYNCHED = u'''\
Against their protests, the villagers lynch {0}, who it turns out is a {1}.'''

WIN_VILLAGERS = u'''\
The wolves are all dead... Villagers win! \
Villagers chop up and bbq the wolves and have a hearty meal.'''

WIN_WOLVES = u'''\
There are the same number of wolves as villagers. \
Wolves eat everyone, and win.'''


def _convert_votes(votes):
    """Convert a mapping voters -> votee to a dict votecount -> votees."""
    votecount = defaultdict(int)
    for victim in votes.itervalues():
        votecount[victim] += 1
    votes_to_victims = defaultdict(set)
    for victim, votes in votecount.iteritems():
        votes_to_victims[votes].add(victim)
    return votes_to_victims

class WolfBotHandler(commands.CommandHandler, commands.ConfigurableMixin):

    implements(ibot.IJoinHandler, ibot.ILeftHandler, ibot.IUserLeftHandler,
               ibot.IMessageHandler)

    configurables = {
        u'channel': (
            u'Channel we play in', 'channel',
            u'%r', commands.bytestring),
        }

    # We override the normal commandset attribute based on phase.
    # This means our individual commands do not need to check game phase.
    phase_to_commands = {
        PHASE_NONE: frozenset([u'join']),
        PHASE_STARTING: frozenset([u'start', u'join', u'del',
                                   u'fstart', u'fjoin', u'fdel',
                                   u'stats', u'ping', u'pong']),
        PHASE_NIGHT: frozenset([u'goat', u'del', u'fdel', u'stats',
                                u'kill', u'see', u'visit']),
        PHASE_LYNCH: frozenset([u'goat', u'del', u'fdel', u'stats', u'votes',
                                u'lynch', u'shoot']),
        }

    maxplayers = 25
    minstarttime = 90 # seconds
    night_timeout_base = 80 # seconds
    night_timeout_per_wolf = 40
    lynch_ping_timeout = 120 # seconds
    lynch_del_timeout = 60 # seconds
    pingtime = 90 # seconds

    _force_roles = False # derandomize role assignment
    # Tests can override this.
    randint = staticmethod(random.randint)

    def __init__(self, _clock=None):
        commands.CommandHandler.__init__(self)
        if _clock is None: # pragma: no cover
            from twisted.internet import reactor as _clock
        self.clock = _clock
        self.timer = None
        self.channel = None
        self.chanobj = None
        self.idlers = set()
        self._setPhase(PHASE_NONE)
        self.lastping = self.clock.seconds() - self.pingtime
        self.pinger = None

    def joined(self, channel):
        if channel.name == self.channel:
            self.chanobj = channel

    def left(self, channel):
        if channel is self.chanobj:
            self.chanobj = None

    def chanmsg(self, msg):
        if self.chanobj is not None:
            self.chanobj.msg(commands.bold(msg))

    def _userlist(self, user, source=None):
        if source is None:
            source = user
        source.msgTo(
            user,
            u'{0:d} {1}: {2}'.format(
                len(self.players),
                u'player' if len(self.players) == 1 else u'players',
                u', '.join(
                    u'(w) {0}'.format(player.uname)
                    if user in self.wolves and player in self.wolves
                    and user is source
                    else player.uname
                    for player in sorted(
                        self.players, key=operator.attrgetter('uname')))))

    def _initNight(self):
        self.harlottargets = {}
        self.wolfvotes = {}
        self.seersdone = set()

    def _initDay(self):
        self.lynchvotes = {}
        self.nonvoters = set()

    def _setPhase(self, phase):
        self.phase = phase
        self.commandset = self.phase_to_commands[phase]
        if self.timer is not None:
            self.timer.cancel()
            self.timer = None

        if phase == PHASE_NONE:
            self.players = set()
            self.roles = {}
            self.wolves = set()
            self.harlots = set()
            self.seers = set()
            self.idiots = set()
            self.drunks = set()
            self.bullets = {}
            self._initNight()
            self._initDay()
        elif phase == PHASE_NIGHT:
            self.chanmsg(NOW_NIGHT)
            self._initNight()

            for wolf in self.wolves:
                wolf.msg(WOLF_KILL)
            for seer in self.seers:
                seer.msg(SEER_SEE)
            for harlot in self.harlots:
                harlot.msg(HARLOT_VISIT)

            for user in self.seers | self.harlots | self.wolves:
                self._userlist(user)

            self.timer = self.clock.callLater(
                self.night_timeout_base +
                self.night_timeout_per_wolf * len(self.wolves),
                self._finishNight)
        elif phase == PHASE_LYNCH:
            self.chanmsg(NOW_LYNCH)
            self._initDay()
            self.idlers = set(self.players)

            self.timer = self.clock.callLater(self.lynch_ping_timeout,
                                              self._lynchPing)

    def handle(self, user, source, command, args):
        if self.chanobj is None:
            # Yeah, that's not going to work.
            source.msgTo(user, u'I am not on my control channel!')
            return None
        return commands.CommandHandler.handle(
            self, user, source, command, args)

    @commands.args(commands.string)
    def handle_goat(self, user, source, target):
        """goat <target>: have a goat kick target."""
        # XXX should this restrict the target to nicks?
        self.chanmsg(GOAT.format(target, user.uname))

    @commands.args()
    @defer.inlineCallbacks
    def handle_ping(self, user, source):
        now = self.clock.seconds()
        if self.lastping + self.pingtime > now:
            source.msgTo(user, PING_TOO_FAST)
            return

        self.lastping = now
        self.pinger = user

        # Update away status.
        yield self.chanobj.who()

        self.chanmsg(u'{0} says PING!!! {1}'.format(
                user.uname, u' '.join(sorted(
                        lurker.uname
                        for lurker in self.chanobj.users.itervalues()
                        if lurker not in self.players
                        and not lurker.away
                        and lurker is not user))))

    # TODO: there was an "fping" ignoring the timer, but that seems silly.

    @commands.args()
    def handle_pong(self, user, source):
        if self.pinger is None:
            return

        self.chanobj.msgTo(self.pinger,
                           u'{0} says PONG!!!'.format(user.uname))

    def message(self, user, source, message):
        if source is self.chanobj:
            self.idlers.discard(user)

        if user is source and user in self.wolves:
            for wolf in self.wolves - set([user]):
                # TODO: just use msgBytes once that exists
                wolf.msg(u'{0} says: {1}'.format(
                        user.uname, message.decode('ascii', 'replace')))

    def _lynchPing(self):
        if self.idlers:
            self.chanmsg(PING_ALIVE.format(u', '.join(
                        sorted(idler.uname for idler in self.idlers))))
            self.timer = self.clock.callLater(self.lynch_del_timeout,
                                              self._lynchDel)
        else:
            self.timer = None

    def _lynchDel(self):
        self.timer = None
        # We mutate this set from _remove, so need to list() it:
        for idler in list(self.idlers):
            self.chanmsg(KILL_IDLE.format(idler.uname))
            self._remove(idler)

    @commands.args()
    def handle_stats(self, user, source):
        self._userlist(user, source)
        if self.phase not in (PHASE_NONE, PHASE_STARTING):
            source.msgTo(user, STATS.format(wolves=len(self.wolves),
                                            harlots=len(self.harlots),
                                            seers=len(self.seers)))

    @commands.args()
    def handle_votes(self, user, source):
        votes = defaultdict(set)
        for voter, victim in self.lynchvotes.iteritems():
            votes[victim].add(voter)

        msg = [u'Votes:']
        for victim, voters in sorted(votes.iteritems(),
                                     key=lambda t: t[0].uname):
            msg.append(u'{0}: {1} ({2})'.format(
                    victim.uname, len(voters), u' '.join(
                        sorted(voter.uname for voter in voters))))
        source.msgTo(user, u' '.join(msg))
        source.msgTo(user, VOTES.format(
                len(self.players),
                (len(self.players) - len(self.nonvoters)) // 2 + 1,
                len(self.players) - len(self.nonvoters)))

    @commands.args(commands.bytestring)
    def handle_fjoin(self, user, source, target):
        """fjoin <target>: force target into the game."""
        utarget = target.decode('ascii', 'replace')

        targetobj = self.chanobj.users.get(target)
        if targetobj is None:
            source.msgTo(
                user, u'{0} is not in the correct channel. '
                u'Ask them to join {1} first.'.format(
                    utarget, self.chanobj.uname))
        elif targetobj in self.players:
            source.msgTo(user, u'{0} is already in the game.'.format(utarget))
        else:
            self.chanobj.msgTo(
                targetobj,
                u'you have been joined to the game by {0}'.format(user.uname))
            self._join(targetobj)

    @commands.args()
    def handle_join(self, user, source):
        """join: join the game."""
        # This is valid in PHASE_NONE and PHASE_STARTING
        if self.channel not in user.channels:
            source.msgTo(
                user,
                u'You are not in the correct channel. Join {0} first.'.format(
                    self.chanobj.uname))
        elif user in self.players:
            source.msgTo(user, u'you are already in the game')
        elif len(self.players) >= self.maxplayers:
            source.msgTo(user,
                         u'Sorry, but there are too many players already.')
        else:
            if self.phase == PHASE_NONE:
                self.starttime = self.clock.seconds()
                self.chanmsg(STARTED.format(user.uname))
                self._setPhase(PHASE_STARTING)
            self._join(user)

    def _join(self, user):
        self.players.add(user)
        self.chanobj.mode(True, 'v', user.name)
        # possibly announce something here

    @commands.args()
    def handle_start(self, user, source):
        """start: start the game."""
        if user not in self.players:
            source.msgTo(user, u'You are not playing.')
            return

        delay = self.starttime + self.minstarttime - self.clock.seconds()
        if delay > 0:
            # kludge: add 5 to the delay to force rounding up. Avoids
            # weird "0 seconds" waits, and is more useful since
            # "start" after the number mentioned always works.
            source.msgTo(user,
                         WAIT_FOR_PLAYERS.format(int(round(delay + 5, -1))))
            return

        self.handle_fstart(user, source, u'')

    @commands.args()
    def handle_fstart(self, user, source):
        """fstart: like start but without the join delay."""
        if len(self.players) < 4:
            source.msgTo(user, u'Not enough players!')
            return

        self.chanobj.mode(True, 'm')
        self.chanmsg(u'Game is now starting.')

        unassigned = list(self.players)
        random.shuffle(unassigned)
        # Careful here, the code below blows up if there are not
        # enough unassigneds left...
        while len(self.wolves) < len(self.players) * .14:
            wolf = unassigned.pop()
            self.wolves.add(wolf)
            self.roles[wolf] = u'wolf'
            wolf.msg(YOU_ARE_WOLF)

        # Currently there is always exactly one seer.
        # We could add more seers for larger games though.
        seer = unassigned.pop()
        self.seers.add(seer)
        self.roles[seer] = u'seer'
        seer.msg(YOU_ARE_SEER)

        idiot = unassigned.pop()
        self.idiots.add(idiot)
        self.roles[idiot] = u'village idiot'
        drunk = unassigned.pop()
        self.drunks.add(drunk)
        self.roles[drunk] = u'village drunk'

        # The code below contains test-specific hacks: If _force_roles
        # is on one user cannot have multiple roles, and the role is
        # always assigned if the min number of players is available.
        if len(self.players) >= 7:
            harlot = unassigned.pop()
            self.harlots.add(harlot)
            self.roles[harlot] = u'harlot'
            harlot.msg(YOU_ARE_HARLOT)

        if len(self.players) >= 6 and (self._force_roles
                                       or random.random() > .1):
            candidates = self.players - self.wolves
            if self._force_roles:
                # HACK to make the tests easier to write.
                candidates -= self.seers | self.harlots
            shooter = random.choice(list(candidates))
            # I can't be bothered to test all of these:
            if len(self.players) < 8: # pragma: no cover
                bullets = 1
            elif len(self.players) < 10:
                bullets = random.randint(1, 2)
            elif len(self.players) < 13: # pragma: no cover
                bullets = random.randint(1, 3)
            else: # pragma: no cover
                bullets = random.randint(1, 4)

            # Too random to conveniently test.
            if shooter in self.drunks: # pragma: no cover
                bullets *= 3

            self.bullets[shooter] = bullets

            shooter.msg(YOU_ARE_SHOOTER.format(bullets))

        if len(self.wolves) == 1:
            wolf = list(self.wolves)[0]
            self.reveal = u'The wolf was {0}'.format(wolf.uname)
        else:
            self.reveal = u'The wolves were {0}'.format(
                u', '.join(wolf.uname for wolf in self.wolves))

        # assert len(self.seers) == 1, 'game rules changed'
        seer = list(self.seers)[0]
        self.reveal += u' and the seer was {0}'.format(seer.uname)

        # TODO: statsf stuff?
        # There was a 15 seconds sleep here, which I have removed.
        self._setPhase(PHASE_NIGHT)

    def _getTarget(self, user, source, target, purpose):
        targetuser = self.chanobj.users.get(target)
        if targetuser is None or targetuser not in self.players:
            source.msgTo(user, BAD_TARGET.format(purpose))
            return None
        return targetuser

    @commands.args(commands.bytestring)
    @commands.onlyInPrivmsg
    def handle_kill(self, user, source, target):
        if source not in self.wolves:
            return

        targetuser = self._getTarget(user, source, target, u'killing')
        if targetuser is None:
            return

        source.msgTo(user, u'%s is selected.' % (targetuser.uname,))

        self.wolfvotes[user] = targetuser

        self._checkNight()

    @commands.args(commands.bytestring)
    @commands.onlyInPrivmsg
    def handle_visit(self, user, source, target):
        if user not in self.harlots:
            return

        targetuser = self._getTarget(user, source, target, u'visiting')
        if targetuser is None:
            return

        if targetuser is user:
            source.msgTo(user, HARLOT_VISIT_SELF)
            return

        if user in self.harlottargets:
            source.msgTo(user, HARLOT_VISITS_ALREADY)
            return

        self.harlottargets[user] = targetuser
        source.msgTo(user, HARLOT_VISITS_TARGET.format(targetuser.uname))
        targetuser.msg(HARLOT_VISITS_YOU.format(user.uname))

        self._checkNight()

    @commands.args(commands.bytestring)
    @commands.onlyInPrivmsg
    def handle_see(self, user, source, target):
        # Err, there's code in wolfbot.bas to give *wolves* a !see command?
        # Am I reading it wrong?
        if user not in self.seers:
            return

        targetuser = self._getTarget(user, source, target, u'seeing')
        if targetuser is None:
            return

        if user in self.seersdone:
            source.msgTo(user, CANNOT_MULTI_SEE)
            return

        self.seersdone.add(user)
        role = self.roles.get(targetuser, u'normal villager')
        source.msgTo(user, u'{0} is a {1}'.format(targetuser.uname, role))

        self._checkNight()

    def _checkNight(self):
        """Check if the night phase is done."""
        # We currently (intentionally) do not wait for the wolves to agree.
        # We just wait until they have all voted.
        if self.seersdone != self.seers or \
                len(self.harlottargets) != len(self.harlots) or \
                len(self.wolfvotes) != len(self.wolves):
            return

        self.timer.cancel()
        self._finishNight()

    def _finishNight(self):
        self.timer = None
        victim = None
        if self.wolfvotes:
            # Find the victim(s) with the highest number of votes,
            # and pick a random victim from that set.
            # TODO: try different approaches to breaking ties.
            votes_to_victims = _convert_votes(self.wolfvotes)
            victims = votes_to_victims[max(votes_to_victims)]
            victim = random.choice(list(victims))
            # (this checks if it's a harlot out visiting,
            # not if it's the target of a harlot)
            if victim in self.harlottargets:
                self.chanmsg(WOLVES_MISSED_HARLOT)
                victim = None
            else:
                victimname = victim.uname
                victimrole = self.roles.get(victim, u'normal villager')
        if victim is None:
            victimname = u'a young penguin'
            victimrole = u'pet'

        for harlot, target in self.harlottargets.iteritems():
            if target in self.wolves:
                self.chanmsg(HARLOT_VISITS_WOLF.format(harlot.uname))
                self._remove(harlot)
            elif target is victim:
                self.chanmsg(HARLOT_VISITS_VICTIM.format(harlot.uname))
                self._remove(harlot)

        self.chanmsg(WOLVES_KILLED.format(victimname, victimrole))
        if victim is not None:
            self._remove(victim)
        if self.phase:
            self._setPhase(PHASE_LYNCH)

    @commands.args(commands.bytestring)
    def handle_lynch(self, user, source, target):
        if user not in self.players:
            return

        targetuser = self._getTarget(user, source, target, u'lynching')
        if targetuser is None:
            return

        self.chanmsg(u'{0} votes for {1}'.format(user.uname,
                                                 targetuser.uname))
        self.lynchvotes[user] = targetuser
        votes_to_victims = _convert_votes(self.lynchvotes)
        maxvotes = max(votes_to_victims)
        if 2 * maxvotes <= len(self.players) - len(self.nonvoters):
            return

        # Because we currently abort this phase as soon as there is a
        # majority there cannot be a tie if anyone actually has enough
        # votes.
        victims = votes_to_victims[max(votes_to_victims)]
        assert len(victims) == 1, '%r are tied' % (victims,)
        victim = victims.pop()

        self.chanmsg(LYNCHED.format(
                victim.uname, self.roles.get(victim, u'normal villager')))
        self._remove(victim)

        if self.phase:
            self._setPhase(PHASE_NIGHT)

    @commands.args(commands.bytestring)
    def handle_shoot(self, user, source, target):
        bullets = self.bullets.get(user, 0)
        if not bullets:
            return

        targetuser = self._getTarget(user, source, target, u'shooting')
        if targetuser is None:
            return

        rand = self.randint(0, 200 if user in self.drunks else 100)
        # 80% (or 40% if drunk) chance of hitting the target:
        if rand < 80:
            self.chanmsg(SHOOTER_SHOT.format(user.uname, targetuser.uname))
            self.bullets[user] -= 1
            if targetuser in self.wolves:
                self.chanmsg(SHOOTER_SHOT_WOLF.format(targetuser.uname))
                self._remove(targetuser)
            elif rand > 15:
                self.chanmsg(SHOOTER_SHOT_VILLAGER.format(targetuser.uname))
                self.nonvoters.add(targetuser)
                self.lynchvotes.pop(targetuser, None)
                # XXX I suspect the game can get stuck here
                # (we need to trigger a re-check of lynch votes)
            else:
                self.chanmsg(SHOOTER_KILLED_VILLAGER.format(targetuser.uname,
                                                            user.uname))
                self._remove(targetuser)
        elif rand < 95:
            self.chanmsg(SHOOTER_MISSED.format(user.uname))
            self.bullets[user] -= 1
        elif rand < 125:
            self.chanmsg(SHOOTER_KILLED_SELF.format(user.uname))
            self._remove(user)
        elif rand < 175:
            # Yes, this matches the rand < 95 case.
            # Both the "missed" and "killed self" range are wider if
            # the shooter is drunk.
            self.chanmsg(SHOOTER_MISSED.format(user.uname))
            self.bullets[user] -= 1
        else:
            self.chanmsg(SHOOTER_SHOT_SELF.format(user.uname))
            self.bullets[user] -= 1
            self.nonvoters.add(user)
            self.lynchvotes.pop(user, None)

    @commands.args()
    def handle_del(self, user, source):
        if user in self.players:
            self._del(user)

    @commands.args(commands.bytestring)
    def handle_fdel(self, user, source, target):
        targetuser = self._getTarget(user, source, target, u'removing')
        if targetuser is not None:
            self._del(targetuser)

    def userLeft(self, user, channel):
        if user is self.pinger:
            self.pinger = None

        if channel is self.chanobj and user in self.players:
            self._del(user, True)

    def _del(self, user, left=False):
        if self.phase != PHASE_STARTING:
            reason = random.choice([
                    u"a poison berry",
                    u"an angry goat",
                    u"zombies.",
                    u"rabid squirrels.",
                    u"sushi.",
                    u"dropping a hammer on their nose.",
                    u"a grue.",
                    u"dysentery.",
                    ])
            self.chanmsg(u'{0} was killed by {1}.'.format(user.uname, reason))
            role = self.roles.get(user)
            if role is not None:
                self.chanmsg(u'It appears they were a {0}.'.format(role))

        self._remove(user, left)

    def _remove(self, user, left=False):
        self.players.discard(user)
        self.idlers.discard(user)
        self.roles.pop(user, None)
        self.wolves.discard(user)
        self.harlots.discard(user)
        self.seers.discard(user)
        self.idiots.discard(user)
        self.drunks.discard(user)
        self.bullets.pop(user, None)
        self.harlottargets = dict((k, v)
                                  for k, v in self.harlottargets.iteritems()
                                  if k is not user and v is not user)
        self.wolfvotes = dict((k, v) for k, v in self.wolfvotes.iteritems()
                              if k is not user and v is not user)
        self.lynchvotes = dict((k, v) for k, v in self.lynchvotes.iteritems()
                               if k is not user and v is not user)

        if not left:
            self.chanobj.mode(False, 'v', user.name)

        if self.phase == PHASE_STARTING:
            if not self.players:
                self._setPhase(PHASE_NONE)
            return

        if not self.wolves:
            self.chanmsg(WIN_VILLAGERS)
        # XXX is taking nonvoters into account here correct?
        elif len(self.players) - len(self.nonvoters) <= 2 * len(self.wolves):
            self.chanmsg(WIN_WOLVES)
        else:
            # Game continues:
            assert self.players
            return
        # Game over:
        self.chanmsg(self.reveal)
        self.chanobj.mode(False, 'm')
        for player in self.players:
            self.chanobj.mode(False, 'v', player.name)
        self._setPhase(PHASE_NONE)
