
"""Fixes up unicode handling in pyyaml."""


# pyyaml unicode handling (in python 2) is... strange.
#
# yaml.dump(u'\N{SNOWMAN}') => '"\\u2603"\n' (ok)
# yaml.dump(u'snowman')     => "!!python/unicode 'snowman'\n" (um, no).
#
# yaml.safe_dump('\xff')    => '!!binary |\n  /w==\n' (great).
# yaml.safe_dump('a')       => 'a\n...\n' (ummm)
# yaml.safe_dump('\xe2\x98\x83') => '"\\u2603"\n' (no!)
#
# Most of this is because yaml.load and yaml.safe_load (unfortunately)
# try to return str instead of unicode for ascii input.
# I much prefer the python 3-style behavior of always giving me unicode
# (except when the input is bytes/binary).


from __future__ import absolute_import, with_statement, division

import yaml


class SaneDumper(yaml.SafeDumper):

    def represent_str(self, data):
        return self.represent_scalar(u'tag:yaml.org,2002:binary',
                                     data.encode('base64'),
                                     style='|')

SaneDumper.add_representer(str, SaneDumper.represent_str)


class SaneLoader(yaml.SafeLoader):
    pass

SaneLoader.add_constructor(u'tag:yaml.org,2002:str',
                           SaneLoader.construct_scalar)


def dump(data, stream=None, **kwargs):
    return yaml.dump(data, stream, Dumper=SaneDumper, **kwargs)


def load(stream):
    return yaml.load(stream, SaneLoader)
